/*                
 *                        Licensing Notification
 *
 * Permission to use, reproduce, prepare derivative works, and to redistribute
 * to others this software and its documentation is hereby granted, provided
 * that this notice is retained thereon and on all copies or modifications. UChicago
 * Argonne, LLC, Tech-X Corporation, and all other contributors make no representations
 * as to the suitability and operability of this software for any purpose. It is
 * provided "as is" without express or implied warranty.
 *
 */

#ifndef OPERATOR_T_H
#define OPERATOR_T_H

// -----------------------------------------------------------------------------

#ifdef __CUDACC__
#define OPERATOR_HOST_DEVICE __host__ __device__
#else
#define OPERATOR_HOST_DEVICE
#endif

#define OPERATOR_T(name, op_symb)                                         \
struct name                                                               \
{                                                                         \
	template<typename value_type_out, typename value_type_in>             \
	OPERATOR_HOST_DEVICE                                                  \
	static inline void op(value_type_out& out, const value_type_in& in)   \
	{                                                                     \
		out op_symb in;                                                   \
	}                                                                     \
};

OPERATOR_T(operator_assign,  =)
OPERATOR_T(operator_add,    +=)
OPERATOR_T(operator_sub,    -=)
OPERATOR_T(operator_mult,   *=)
OPERATOR_T(operator_div,    /=)

#ifdef __CUDACC__
#undef OPERATOR_HOST_DEVICE
#endif

// -----------------------------------------------------------------------------

#endif // OPERATOR_T_H
